/* Programming activity: Exceptions and File I/O (SOLUTION)
 *
 * Goal: compute the overlapping friends in the lists of two people.
 * The overlapping friend have been contacted by both people a minimum 
 * number of times.
 * 
 * Part 1, the program currently crashes if not given 4 command line arguments.
 * It also crashes if the 4th argument isn't an int.  Add code to print a 
 * helpful message in both cases before exiting.  
 * 
 *    % java CommonFriends
 *    CommonFriends <person1 in file> <person2 in file> <out file> <min msgs>
 * 
 *    % java CommonFriends person1.txt person2.txt common.txt -12.5
 *    Minimum number of messages must be an integer!
 *  
 * Part 2, add code to read the input from person 1.  The input file consists
 * of the name of a friend followed by the integer count of messages sent
 * to that friend.
 * 
 *    % java CommonFriends person1.txt person2.txt common.txt 2
 *    Person1 has 95 friends that they have contacted 2 or more times.
 * 
 *    % java CommonFriends person1.txt person2.txt common.txt 20
 *    Person1 has 4 friends that they have contacted 20 or more times.
 *    
 * Part 3, add code to read the input from person 2 and output the
 * overlapping friends to the output file.
 * 
 *    % java CommonFriends person1.txt person2.txt common.txt 2
 *    Person1 has 95 friends that they have contacted 2 or more times.
 *    
 *    % more common.txt
 *    Donald
 *    Millicent
 *    Kathryne
 *    Marian
 */

import java.util.*;
import java.io.*;

public class CommonFriends
{
    public static void main(String[] args)
    {
        // Make sure the program got at least 4 command line arguments.
        // If not, print a helpful message and exit.
        // TODO 1: add an if statement that protects against insufficient command line input
        if (args.length < 4)
        {
            System.out.println("<person1 in file> <person2 in file> <out file> <min msgs>");
            return;
        }
        
        String inFilename1 = args[0];
        String inFilename2 = args[1];
        String outFilename = args[2];        
        
        int minMessages = 0;

        // TODO 1: if minimum messages can't be converted to an int, print a message and exit.
        // This will require surrounding the following line with a try-catch block.
        // NumberFormatException is the exception type thrown when parseInt gets bad text.
        try
        {
            minMessages = Integer.parseInt(args[3]);
        }
        catch (NumberFormatException e)
        {
            System.out.println("Minimum number of messages must be an integer!");
        }
        
        // First we'll read in all the friends in the first person's list.
        // Store in an ArrayList all friends that person 1 contacted the minimum number of times. 
        ArrayList<String> friends = new ArrayList<String>();

        // Try and open up the first input file        
        Scanner scanner;
        try
        {
            // TODO 2: replace null to open the first input file
            scanner = new Scanner(new File(inFilename1));
        }
        // TODO 2: replace Exception with the more specific type FileNotFoundException
        catch (Exception e)
        {
            System.out.println("Failed to open input file 1: " + inFilename1);
            return;
        }
        
        // Now read in the data which consists of a name followed by a integer message count
        while (scanner.hasNext())
        {
            // Read in the friend's name
            // TODO 2: read in a string using the scanner
            String friend = scanner.next();
            
            // Read in the message count
            // TODO 2: read in an int using the scanner
            int messages = scanner.nextInt();

            // If this friend meets the minimum number of messages, add to the ArrayList
            // TODO 2: add conditional to do the above
            if (messages >= minMessages)
                friends.add(friend);
        }
        
        // Always good to cleanup by closing the scanner
        // TODO 2: cleanup since we are done with the first input file

        // Print a message saying how many friends person1 had meeting the minimum, for example:
        System.out.printf("Person1 has %d friends that they have contacted %d or more times\n", friends.size(), minMessages);
        
        // Now go through all the friends in the second person's list.
        // Output names to the output file if:
        //  1) the friend was also in the first person's list
        //  2) person 2 had contacted this friend the minimum number of times
        
        // Try and open up the second input file
        try
        {
            // TODO 3: replace null to open up second input file
            scanner = new Scanner(new File(inFilename2));
        }
        // TODO 3: replace Exception with the more specific type FileNotFoundException        
        catch (Exception e)
        {
            System.out.println("Failed to open input file 2: " + inFilename2);
            return;
        }
        
        // Next establish we can open up the output file
        PrintWriter writer;
        try
        {          
            // TODO 3: replace null with code to open up the output file
            writer = new PrintWriter(outFilename);
        }
        // TODO 3: replace Exception with the more specific type FileNotFoundException
        catch (Exception e)
        {
            System.out.println("Failed to open output file: " + outFilename);
            scanner.close();
            return;
        }
        
        // TODO 3: fix the while condition so it only loops while input is available
        while (scanner.hasNext())
        {
            String friend = scanner.next();
            int messages = scanner.nextInt();

            // Output to file if this friend count meets the minimum and if
            // the contact is also in the ArrayList.
            // TODO 3: replace if condition to do the above.
            if (messages >= minMessages && friends.contains(friend))
            {
                // TODO 3: print the friend variable to the output file
                writer.println(friend);
            }
        }
        
        // TODO 3: cleanup both the input and output file 
        scanner.close();
        writer.close();
    }
}
